import os

from pyramid.config import Configurator
from pyramid.session import SignedCookieSessionFactory
from barker.renderers import json_renderer, CSVRenderer
from barker.transactional_view_deriver import transactional_view


def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    if 'sqlalchemy.url' not in settings:
        DB_NAME = os.environ['DB_NAME']
        DB_USER = os.environ['DB_USER']
        DB_PASS = os.environ['DB_PASS']
        DB_URI = 'postgresql://{0}:{1}@postgres:5432/{2}'.format(
            DB_USER, DB_PASS, DB_NAME
        )
        settings['sqlalchemy.url'] = DB_URI

    SECRET_KEY = os.environ.get('SECRET_KEY', settings.get('secret_key', ''))
    session_factory = SignedCookieSessionFactory(SECRET_KEY, timeout=None)

    config = Configurator(
        settings=settings,
        session_factory=session_factory
    )
    config.include('.security')
    config.include('.models')
    config.include('.routes')
    config.add_view_deriver(transactional_view)
    config.add_renderer(name='json', factory=json_renderer)
    config.add_renderer(name='csv', factory=CSVRenderer)
    config.scan()

    return config.make_wsgi_app()
