import uuid

from pyramid.authentication import AuthTktAuthenticationPolicy
from barker.security.permission_authorization_policy import PermissionAuthorizationPolicy
from ..models.auth import User


def get_user(request):
    user_id = request.unauthenticated_userid
    if user_id is not None:
        user = request.dbsession.query(User).get(user_id)
        return user


def groupfinder(user_id, request):
    if 'perms' in request.session:
        perms = request.session['perms']
    else:
        perms = []
        user = request.dbsession.query(User).filter(User.id == uuid.UUID(user_id)).one()
        for role in user.roles:
            for perm in role.permissions:
                perms.append(perm.name)
        perms = sorted(f7(perms))
        request.session['perms'] = perms
    return perms


def f7(seq):
    seen = set()
    seen_add = seen.add
    return [x for x in seq if x not in seen and not seen_add(x)]


def includeme(config):
    authn_policy = AuthTktAuthenticationPolicy('barker', timeout=900, reissue_time=90, callback=groupfinder)
    config.set_authentication_policy(authn_policy)
    config.set_authorization_policy(PermissionAuthorizationPolicy())
    config.add_request_method(get_user, 'user', reify=True)
