from decimal import Decimal
import uuid
import re

import pkg_resources

from barker.exceptions import ValidationFailure
from pyramid.response import Response, FileResponse
from pyramid.view import forbidden_view_config, exception_view_config, view_config


@view_config(route_name="home")
@view_config(request_method="GET", route_name="login")
def home(request):
    package, resource = "barker:static/index.html".split(":", 1)
    file = pkg_resources.resource_filename(package, resource)
    return FileResponse(file, request=request)


@forbidden_view_config(renderer='json')
def forbidden(request):
    response = Response()
    response.status_int = 403 if request.authenticated_userid is not None else 401
    response.content_type = 'application/json'
    response.json = {'Error': 'Forbidden'}
    return response


@exception_view_config(ValidationFailure)
def failed_validation(exc, request):
    response = Response()
    response.status_int = 500
    response.content_type = 'application/json'
    response.json = {'Error': "Failed validation: {0!s}".format(exc.message)}
    return response


def to_uuid(value):
    p = re.compile('^[A-Fa-f0-9]{8}-[A-Fa-f0-9]{4}-[A-Fa-f0-9]{4}-[A-Fa-f0-9]{4}-[A-Fa-f0-9]{12}$')
    return uuid.UUID(value) if p.match(value) else None


def to_decimal(value, default=0):
    import re
    _parser = re.compile(r"""        # A numeric string consists of:
        (?P<sign>[-+])?              # an optional sign, followed by either...
        (
            (?=\d|\.\d)              # ...a number (with at least one digit)
            (?P<int>\d*)             # having a (possibly empty) integer part
            (\.(?P<frac>\d*))?       # followed by an optional fractional part
        |
            (?P<signal>s)?           # ...an (optionally signaling)
            NaN                      # NaN
            (?P<diag>\d*)            # with (possibly empty) diagnostic info.
        )
        \Z
    """, re.VERBOSE | re.IGNORECASE).match
    return Decimal(value) if _parser(value.strip()) is not None else default
