import uuid

import transaction
from pyramid.view import view_config

from barker.models import Customer


@view_config(request_method='PUT', route_name='customer', renderer='json', permission='Customers', trans=True)
def save(request):
    json = request.json_body
    item = Customer(json['company'], json['name'], json['phone'], json['address'], json['important'])
    request.dbsession.add(item)
    transaction.commit()
    item = request.dbsession.query(Customer).filter(Customer.id == item.id).first()
    return customer_info(item)


@view_config(request_method='POST', route_name='customer_id', renderer='json', permission='Customers', trans=True)
def update(request):
    json = request.json_body
    item = request.dbsession.query(Customer).filter(Customer.id == uuid.UUID(request.matchdict['id'])).first()
    item.company = json['company']
    item.name = json['name']
    item.phone = json['phone']
    item.address = json['address']
    item.is_important = json['important']
    transaction.commit()
    item = request.dbsession.query(Customer).filter(Customer.id == item.id).first()
    return customer_info(item)


@view_config(request_method='DELETE', route_name='customer_id', renderer='json', permission='Customers', trans=True)
def delete(request):
    item = request.dbsession.query(Customer).filter(Customer.id == uuid.UUID(request.matchdict['id'])).first()
    request.dbsession.delete(item)
    transaction.commit()
    return {}


@view_config(request_method='GET', route_name='customer_id', renderer='json', permission='Authenticated')
def show_id(request):
    id = uuid.UUID(request.matchdict['id'])
    item = request.dbsession.query(Customer).filter(Customer.id == id).first()
    return customer_info(item)


@view_config(request_method='GET', route_name='customer_list', renderer='json', permission='Authenticated')
def show_list(request):
    list = request.dbsession.query(Customer).order_by(Customer.name).all()
    customers = []
    for item in list:
        customers.append({
            'id': item.id,
            'company': item.company,
            'name': item.name,
            'address': item.address,
            'important': item.is_important,
            'phone': item.phone,
            'remarks': ''
        })
    return customers


def customer_info(item):
    return {
        'id': item.id,
        'company': item.company,
        'name': item.name,
        'address': item.address,
        'important': item.is_important,
        'phone': item.phone,
        'remarks': ''
    }
