import uuid

import transaction

from pyramid.httpexceptions import HTTPFound
from pyramid.response import Response
from pyramid.security import forget, remember
from pyramid.view import view_config

from barker.models import User
from barker.security import f7


@view_config(route_name='logout')
def logout(request):
    request.session.invalidate()
    headers = forget(request)
    return HTTPFound(location=request.route_url('home'), headers=headers)


@view_config(request_method="POST", route_name="logout", renderer="json")
def api_logout(request):
    request.session.invalidate()
    request.response.headers = forget(request)
    return {"isAuthenticated": False, "perms": {}}


@view_config(request_method='POST', route_name='v1_login', renderer='json', trans=True)
def login(request):
    username = request.json_body.get('name', None)
    password = request.json_body.get('password', None)
    found, user = User.auth(username, password, request.dbsession)

    if found:
        headers = remember(request, str(user.id))
        auth = {
            'id': str(user.id),
            'name': user.name,
            'lockedOut': user.locked_out,
            'isAuthenticated': True,
            'perms': sorted(f7([p.name for r in user.roles for p in r.permissions]))
        }
        request.response.headers = headers
        request.response.content_type = 'application/json'
        request.response.json = auth
        return request.response
    elif not found:
        response = Response()
        response.status_int = 401
        response.content_type = 'application/json'
        response.json = {'Error': 'Login Failed'}
        transaction.abort()
        return response


def get_permissions(user):
    perms = []
    for role in user.roles:
        for perm in role.permissions:
            perms.append({'PermissionID': str(perm.id), 'Name': perm.name})
    return sorted({p['PermissionID']: p for p in perms}.values(), key=lambda i: i['Name'])


@view_config(route_name='v1_auth', renderer='json')
def user_permission(request):
    user_id = request.authenticated_userid
    if user_id is None:
        auth = {'isAuthenticated': False, 'perms': {}}
    elif 'auth' in request.session:
        auth = request.session['auth']
    else:
        user = request.dbsession.query(User).filter(User.id == uuid.UUID(user_id)).one()
        auth = {
            'id': user.id,
            'name': user.name,
            'isAuthenticated': True,
            'perms': sorted(f7([p.name for r in user.roles for p in r.permissions]))
        }
        request.session['auth'] = auth
    return auth
