import uuid
import transaction
from pyramid.view import view_config

from barker.models import Printer
from barker.models.validation_exception import ValidationError


@view_config(
    request_method="POST",
    route_name="v1_printers_new",
    renderer="json",
    permission="Printers",
    trans=True,
)
def save(request):
    json = request.json_body
    name = json.get("name", "").strip()
    if name == "":
        raise ValidationError("Name cannot be blank")
    address = json.get("address", "").strip()
    if address == "":
        raise ValidationError("Address cannot be blank")
    cut_code = json.get("cutCode", "")
    item = Printer(name, address, cut_code)
    request.dbsession.add(item)
    transaction.commit()
    return printer_info(item.id, request.dbsession)


@view_config(
    request_method="PUT",
    route_name="v1_printers_id",
    renderer="json",
    permission="Printers",
    trans=True,
)
def update(request):
    json = request.json_body
    item = (
        request.dbsession.query(Printer)
        .filter(Printer.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )
    item.name = json.get("name", "").strip()
    if item.name == "":
        raise ValidationError("Name cannot be blank")
    item.address = json.get("address", "").strip()
    if item.address == "":
        raise ValidationError("Address cannot be blank")
    item.cut_code = json.get("cutCode", "")
    transaction.commit()
    return printer_info(item.id, request.dbsession)


@view_config(
    request_method="DELETE",
    route_name="v1_printers_id",
    renderer="json",
    permission="Printers",
    trans=True,
)
def delete(request):
    item = (
        request.dbsession.query(Printer)
        .filter(Printer.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )
    if item.is_fixture:
        raise ValidationError(
            "{0} is a fixture and cannot be edited or deleted.".format(item.full_name)
        )
    request.dbsession.delete(item)
    transaction.commit()
    return printer_info(None, request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_printers_new",
    renderer="json",
    permission="Authenticated",
)
def show_blank(request):
    return printer_info(None, request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_printers_id",
    renderer="json",
    permission="Authenticated",
)
def show_id(request):
    return printer_info(uuid.UUID(request.matchdict["id"]), request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_printers_list",
    renderer="json",
    permission="Authenticated",
)
def show_list(request):
    list_ = request.dbsession.query(Printer).order_by(Printer.name).all()
    printers = []
    for item in list_:
        printers.append(
            {
                "id": item.id,
                "name": item.name,
                "address": item.address,
                "cutCode": item.cut_code,
            }
        )
    return printers


def printer_info(id_, dbsession):
    if id_ is None:
        printer = {"name": "", "address": "", "cutCode": ""}
    else:
        item = dbsession.query(Printer).filter(Printer.id == id_).first()
        printer = {
            "id": item.id,
            "name": item.name,
            "address": item.address,
            "cutCode": item.cut_code,
        }
    return printer
