import datetime

from pyramid.httpexceptions import HTTPForbidden
from pyramid.view import view_config
from sqlalchemy import func

from barker.models import Inventory, Kot, Product, Settlement, SettleOption, Voucher


@view_config(request_method='GET', route_name='beer_consumption', renderer='json', permission='Beer Consumption',
             request_param=('s', 'f'))
def beer_consumption(request):
    start_date = datetime.datetime.strptime(request.GET['s'], '%d-%b-%Y %H:%M')
    finish_date = datetime.datetime.strptime(request.GET['f'], '%d-%b-%Y %H:%M')

    if (datetime.date.today() - start_date.date()).days > 5 and 'Accounts Audit' not in request.effective_principals:
        raise HTTPForbidden("Accounts Audit")

    list = request.dbsession.query(
        Voucher.date, Product.id, Product.name, func.sum(Inventory.quantity * Product.quantity)
    ).join(Voucher.kots).join(Kot.inventories).join(Inventory.product).filter(
        Voucher.date >= start_date,
        Voucher.date <= finish_date,
        Voucher.is_void == False,
        Voucher.settlements.any(Settlement.settled.in_([
            SettleOption.CASH(),
            SettleOption.CREDIT_CARD(),
            SettleOption.BILL_TO_COMPANY(),
            SettleOption.STAFF(),
            SettleOption.NO_CHARGE()
        ]))
    ).group_by(
        Voucher.date, Product.id, Product.name
    ).order_by(
        Voucher.date, Product.id, Product.name
    ).all()
    return [{
                'Date': i[0].strftime('%d-%b-%Y %H:%M:%S'),
                'ProductID': str(i[1]),
                'Name': i[2],
                'Quantity': i[3]
            } for i in list if i[3] != 0]
