import datetime

from pyramid.httpexceptions import HTTPForbidden
from pyramid.view import view_config
from sqlalchemy.orm import joinedload

from barker.models import Settlement, SettleOption, Voucher


@view_config(request_method='GET', route_name='bill_details', renderer='json', permission='Bill Details',
             request_param=('s', 'f'))
def bill_details(request):
    start_date = datetime.datetime.strptime(request.GET['s'], '%d-%b-%Y %H:%M')
    finish_date = datetime.datetime.strptime(request.GET['f'], '%d-%b-%Y %H:%M')

    if (datetime.date.today() - start_date.date()).days > 5 and 'Accounts Audit' not in request.effective_principals:
        raise HTTPForbidden("Accounts Audit")

    # TODO: Should be left outer join incase some bill is missing the settlements
    vouchers = request.dbsession.query(
        Voucher
    ).options(
        joinedload(Voucher.settlements, innerjoin=True).joinedload(Settlement.settle_option, innerjoin=True),
        joinedload(Voucher.customer, innerjoin=True)
    ).filter(
        Voucher.date >= start_date,
        Voucher.date <= finish_date
    ).order_by(
        Voucher.voucher_type
    ).order_by(
        Voucher.bill_id
    ).all()

    report = []
    for item in vouchers:
        if item.is_void:
            report.append({
                'Date': item.date.strftime('%d-%b-%Y %H:%M:%S'),
                'BillID': item.full_bill_id,
                'Settlement': "Void: {0}".format(item.void_reason),
                'Amount': round(next(
                    s.amount for s in item.settlements if s.settled == SettleOption.AMOUNT()
                ) * -1, 2)
            })
        else:
            for so in (so for so in item.settlements if so.settle_option.show_in_choices):
                report.append({
                    'Date': item.date.strftime('%d-%b-%Y %H:%M:%S'),
                    'BillID': item.full_bill_id,
                    'Settlement': so.settle_option.name,
                    'Amount': round(so.amount, 2)
                })
    return report
