import uuid

from datetime import datetime, timedelta
from barker.models import Voucher, User, Settlement
from pyramid.view import view_config
from sqlalchemy.orm import joinedload

from barker.views.reports import get_start_date, get_finish_date

__author__ = "tanshu"


@view_config(
    request_method="GET",
    route_name="v1_active_cashiers",
    renderer="json",
    permission="Cashier Checkout",
)
def active_cashiers(request):
    start_date = get_start_date(request.GET.get("s", None))
    finish_date = get_finish_date(request.GET.get("f", None))

    user_ids = (
        request.dbsession.query(Voucher.user_id)
        .filter(Voucher.date >= start_date, Voucher.date <= finish_date)
        .distinct()
    )

    users = (
        request.dbsession.query(User)
        .filter(User.id.in_(user_ids))
        .order_by(User.name)
        .all()
    )

    return [{"id": u.id, "name": u.name} for u in users]


@view_config(
    request_method="GET",
    route_name="v1_checkout_blank",
    renderer="json",
    permission="Cashier Checkout",
)
def blank_out(request):
    return {
        "startDate": datetime.today().strftime("%d-%b-%Y"),
        "finishDate": datetime.today().strftime("%d-%b-%Y"),
        "user": {"id": ""},
        "amounts": [],
        "info": [],
    }


@view_config(
    request_method="GET",
    route_name="v1_checkout_id",
    renderer="json",
    permission="Cashier Checkout",
    request_param=("s", "f"),
)
def check_me_out(request):
    id_ = uuid.UUID(request.matchdict["id"])
    start_date = get_start_date(request.GET.get("s", None))
    finish_date = get_finish_date(request.GET.get("f", None))

    vouchers = (
        request.dbsession.query(Voucher)
        .options(
            joinedload(Voucher.settlements, innerjoin=True).joinedload(
                Settlement.settle_option, innerjoin=True
            )
        )
        .filter(
            Voucher.date >= start_date,
            Voucher.date <= finish_date,
            Voucher.user_id == id_,
        )
        .order_by(Voucher.voucher_type)
        .order_by(Voucher.bill_id)
        .all()
    )

    info = {}
    amounts = {}
    for item in vouchers:
        for so in (so for so in item.settlements if so.settle_option.show_in_choices):
            if so.settled not in info:
                info[so.settled] = []
                amounts[so.settle_option.name] = 0
            info[so.settled].append(
                {
                    "date": item.date.strftime("%d-%b-%Y %H:%M:%S"),
                    "billId": item.full_bill_id,
                    "customer": item.customer.name if item.customer is not None else "",
                    "amount": so.amount,
                }
            )
            amounts[so.settle_option.name] += so.amount
    return {
        "startDate": start_date.date().strftime("%d-%b-%Y"),
        "finishDate": (finish_date - timedelta(days=1)).date().strftime("%d-%b-%Y"),
        "user": {"id": id_},
        "amounts": [{"name": key, "amount": value} for key, value in amounts.items()],
        "info": info,
    }
