import datetime

from pyramid.httpexceptions import HTTPForbidden
from pyramid.view import view_config
from sqlalchemy import func

from barker.models import Inventory, Kot, Product, MenuCategory, Settlement, SettleOption, Voucher


@view_config(request_method='GET', route_name='quantity_sold', renderer='json', permission='Sales Detail',
             request_param=('s', 'f'))
def quantity_sold(request):
    start_date = datetime.datetime.strptime(request.GET['s'], '%d-%b-%Y %H:%M')
    finish_date = datetime.datetime.strptime(request.GET['f'], '%d-%b-%Y %H:%M')

    if (datetime.date.today() - start_date.date()).days > 5 and 'Accounts Audit' not in request.effective_principals:
        raise HTTPForbidden("Accounts Audit")

    sold = q(start_date, finish_date, [
        SettleOption.CASH(),
        SettleOption.CREDIT_CARD(),
        SettleOption.BILL_TO_COMPANY(),
        SettleOption.STAFF()
    ], request.dbsession)

    nc = q(start_date, finish_date, [
        SettleOption.NO_CHARGE()
    ], request.dbsession)

    output = [{
                  'ProductID': str(i[0]),
                  'Name': i[1],
                  'IsHappyHour': i[2],
                  'Sale': i[3]
              } for i in sold]
    for id, name, hh, qty in nc:
        i = next((x for x in output if x['ProductID'] == str(id) and x['IsHappyHour'] == hh), None)
        if i is None:
            output.append({'ProductID': str(id), 'Name': name, 'IsHappyHour': hh, 'NC': qty})
        else:
            i['NC'] = qty
    return output


def q(start_date, finish_date, options, dbsession):
    return dbsession.query(
        Product.id, Product.full_name, Inventory.is_happy_hour, func.sum(Inventory.quantity)
    ).join(Voucher.kots).join(Kot.inventories).join(Inventory.product).join(Product.menu_category).filter(
        Voucher.date >= start_date,
        Voucher.date <= finish_date,
        Voucher.is_void == False,
        Voucher.settlements.any(Settlement.settled.in_(options))
    ).group_by(
        Product.id, Product.full_name, Inventory.is_happy_hour, MenuCategory.name, MenuCategory.group_type
    ).order_by(
        MenuCategory.group_type, MenuCategory.name, Product.full_name, Inventory.is_happy_hour
    ).all()
