import datetime

from pyramid.httpexceptions import HTTPForbidden
from pyramid.view import view_config
from sqlalchemy.orm import joinedload

from barker.models import Reprint, Settlement, SettleOption, Voucher


@view_config(request_method='GET', route_name='reprint_report', renderer='json',
             permission='Void or Reprinted Bill Report', request_param=('s', 'f'))
def reprint_report(request):
    start_date = datetime.datetime.strptime(request.GET['s'], '%d-%b-%Y %H:%M')
    finish_date = datetime.datetime.strptime(request.GET['f'], '%d-%b-%Y %H:%M')

    if (datetime.date.today() - start_date.date()).days > 5 and 'Accounts Audit' not in request.effective_principals:
        raise HTTPForbidden("Accounts Audit")

    # TODO: Should be left outer join in case some bill is missing the settlements
    voids = []
    void_q = request.dbsession.query(
        Voucher
    ).options(
        joinedload(Voucher.settlements, innerjoin=True).joinedload(Settlement.settle_option, innerjoin=True),
        joinedload(Voucher.customer, innerjoin=True)
    ).filter(
        Voucher.date >= start_date,
        Voucher.date <= finish_date,
        Voucher.is_void == True
    ).order_by(
        Voucher.voucher_type
    ).order_by(
        Voucher.bill_id
    ).all()

    for item in void_q:
        voids.append({
            'Date': item.date.strftime('%d-%b-%Y %H:%M:%S'),
            'BillID': item.full_bill_id,
            'Settlement': "Void: {0}".format(item.void_reason),
            'Amount': round(next(
                s.amount for s in item.settlements if s.settled == SettleOption.AMOUNT()
            ) * -1, 2)
        })

    reprints = []
    reprint_q = request.dbsession.query(Reprint).filter(
        Reprint.date >= start_date,
        Reprint.date <= finish_date
    ).all()

    for item in reprint_q:
        reprints.append({
            'Date': item.date.strftime('%d-%b-%Y %H:%M:%S'),
            'BillID': item.voucher.full_bill_id,
            'Settlement': "Reprinted by {0}".format(item.user.name),
            'Amount': round(next(
                s.amount for s in item.voucher.settlements if s.settled == SettleOption.AMOUNT()
            ) * -1, 2)
        })

    return voids + reprints
