import uuid

import transaction
from pyramid.response import Response
from pyramid.view import view_config

from barker.models import SaleCategory
from barker.models.validation_exception import ValidationError


@view_config(
    request_method="POST",
    route_name="v1_sale_categories_new",
    renderer="json",
    permission="Products",
    trans=True,
)
def save(request):
    json = request.json_body
    name = json.get("name", "").strip()
    if name == "":
        raise ValidationError("Name cannot be blank")
    tax = json.get("tax", None)
    if tax is None:
        raise ValidationError("please choose a tax")
    tax_id = uuid.UUID(tax["id"])
    item = SaleCategory(name, tax_id)
    request.dbsession.add(item)
    transaction.commit()
    return sale_category_info(item.id, request.dbsession)


@view_config(
    request_method="PUT",
    route_name="v1_sale_categories_id",
    renderer="json",
    permission="Products",
    trans=True,
)
def update(request):
    json = request.json_body
    item = (
        request.dbsession.query(SaleCategory)
        .filter(SaleCategory.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )
    if item.is_fixture:
        raise ValidationError(
            "{0} is a fixture and cannot be edited or deleted.".format(item.full_name)
        )
    item.name = json["name"].strip()
    item.tax_id = uuid.UUID(json["tax"]["id"])
    transaction.commit()
    return sale_category_info(item.id, request.dbsession)


@view_config(
    request_method="DELETE",
    route_name="v1_sale_categories_id",
    renderer="json",
    permission="Products",
    trans=True,
)
def delete(request):
    item = (
        request.dbsession.query(SaleCategory)
        .filter(SaleCategory.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )
    if item is None:
        response = Response("Sale Category not Found")
        response.status_int = 500
        return response
    else:
        response = Response("Sale Category deletion not implemented")
        response.status_int = 500
        return response


@view_config(
    request_method="GET",
    route_name="v1_sale_categories_new",
    renderer="json",
    permission="Authenticated",
)
def show_blank(request):
    return sale_category_info(None, request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_sale_categories_id",
    renderer="json",
    permission="Authenticated",
)
def show_id(request):
    return sale_category_info(uuid.UUID(request.matchdict["id"]), request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_sale_categories_list",
    renderer="json",
    permission="Authenticated",
)
def show_list(request):
    list_ = (
        request.dbsession.query(SaleCategory)
        .order_by(SaleCategory.name)
        .all()
    )
    return [sale_category_info(item, request.dbsession) for item in list_]


@view_config(
    request_method="GET",
    route_name="v1_sale_categories_list",
    renderer="json",
    request_param="d",
    permission="Authenticated",
)
def show_list_for_discount(request):
    list_ = (
        request.dbsession.query(SaleCategory)
        .order_by(SaleCategory.name)
        .all()
    )
    return [{"id": item.id, "name": item.name, "discount": 0} for item in list_]


def sale_category_info(item, dbsession):
    if item is None:
        return {
            "name": "",
            "tax": {}
        }
    if type(item) is uuid.UUID:
        item = dbsession.query(SaleCategory).filter(SaleCategory.id == item).first()
    return {
        "id": item.id,
        "name": item.name,
        "tax": {"id": item.tax_id, "name": item.tax.name, "rate": item.tax.rate}
    }
