import uuid
import transaction
from pyramid.view import view_config

from barker.models import Section
from barker.models.validation_exception import ValidationError


@view_config(
    request_method="POST",
    route_name="v1_sections_new",
    renderer="json",
    permission="Sections",
    trans=True,
)
def save(request):
    json = request.json_body
    name = json.get("name", "").strip()
    if name == "":
        raise ValidationError("Name cannot be blank")
    item = Section(name)
    request.dbsession.add(item)
    transaction.commit()
    return section_info(item.id, request.dbsession)


@view_config(
    request_method="PUT",
    route_name="v1_sections_id",
    renderer="json",
    permission="Sections",
    trans=True,
)
def update(request):
    json = request.json_body
    item = (
        request.dbsession.query(Section)
        .filter(Section.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )
    item.name = json.get("name", "").strip()
    if item.name == "":
        raise ValidationError("Name cannot be blank")
    transaction.commit()
    return section_info(item.id, request.dbsession)


@view_config(
    request_method="DELETE",
    route_name="v1_sections_id",
    renderer="json",
    permission="Sections",
    trans=True,
)
def delete(request):
    item = (
        request.dbsession.query(Section)
        .filter(Section.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )
    if item.is_fixture:
        raise ValidationError(
            "{0} is a fixture and cannot be edited or deleted.".format(item.full_name)
        )
    request.dbsession.delete(item)
    transaction.commit()
    return section_info(None, request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_sections_new",
    renderer="json",
    permission="Authenticated",
)
def show_blank(request):
    return section_info(None, request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_sections_id",
    renderer="json",
    permission="Authenticated",
)
def show_id(request):
    return section_info(uuid.UUID(request.matchdict["id"]), request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_sections_list",
    renderer="json",
    permission="Authenticated",
)
def show_list(request):
    return [
        section_info(item, request.dbsession)
        for item in request.dbsession.query(Section).order_by(Section.name).all()
    ]


def section_info(item, dbsession):
    if item is None:
        return {"name": ""}
    if type(item) is uuid.UUID:
        item = dbsession.query(Section).filter(Section.id == item).first()
    return {"id": item.id, "name": item.name}
