import datetime
import uuid
from decimal import Decimal

import transaction
from pyramid.view import view_config
from sqlalchemy import func

from barker.models import (
    Voucher,
    Kot,
    Inventory,
    InventoryModifier,
    VoucherType,
    Product,
    FoodTable,
)
from barker.models.validation_exception import ValidationError
from barker.views.voucher import (
    get_tax,
    do_update_settlements,
    get_bill_id,
    do_update_table,
    check_permissions,
    get_guest_book,
)


@view_config(
    request_method="POST", route_name="v1_vouchers_new", renderer="json", trans=True
)
def save(request):
    json = request.json_body
    now = datetime.datetime.now()
    update_table = request.GET["u"] == "true"
    voucher_type = VoucherType[request.GET["p"]]
    guest_book = get_guest_book(request.GET.get("g", None), request.dbsession)

    table = (
        request.dbsession.query(FoodTable)
        .filter(FoodTable.id == uuid.UUID(json["table"]["id"]))
        .first()
    )

    check_permissions(None, voucher_type, request.effective_principals)

    bill_id = get_bill_id(voucher_type, request.dbsession)
    kot_id = request.dbsession.query(
        func.coalesce(func.max(Voucher.kot_id), 0) + 1
    ).scalar()

    item = Voucher(
        now,
        guest_book.pax if guest_book is not None else table.seats,
        bill_id,
        kot_id,
        json["table"]["id"],
        json["customer"]["id"] if "id" in json["customer"] else None,
        voucher_type,
        uuid.UUID(request.authenticated_userid),
    )
    request.dbsession.add(item)
    for k in json["kots"]:
        code = request.dbsession.query(func.coalesce(func.max(Kot.code), 0) + 1).scalar()
        kot = Kot(item.id, code, item.food_table_id, item.date, item.user_id)
        item.kots.append(kot)
        request.dbsession.add(kot)
        for index, i in enumerate(k["inventories"]):
            product = (
                request.dbsession.query(Product)
                .filter(Product.id == uuid.UUID(i["product"]["id"]))
                .first()
            )
            tax_rate = get_tax(product.sale_category.tax.rate, voucher_type)
            inv = Inventory(
                kot.id,
                product.id,
                round(Decimal(i["quantity"]), 2),
                product.price,
                round(Decimal(i["discount"]), 5),
                i["isHappyHour"],
                product.sale_category.tax_id,
                tax_rate,
                index,
            )
            kot.inventories.append(inv)
            request.dbsession.add(inv)
            for m in i["modifiers"]:
                mod = InventoryModifier(None, uuid.UUID(m["id"]), 0)
                inv.modifiers.append(mod)
                request.dbsession.add(mod)
    do_update_settlements(item, request.dbsession)
    if len(item.kots) == 0 or len(item.kots[0].inventories) == 0:
        raise ValidationError("Please add some products!")

    if update_table:
        do_update_table(item, guest_book, request.dbsession)
    transaction.commit()
    return True
