import uuid

import transaction
from datetime import datetime
from pyramid.view import view_config
from sqlalchemy import func

from barker.models import Voucher, Overview, Kot, Inventory, InventoryModifier
from barker.models.validation_exception import ValidationError
from barker.views.voucher import get_bill_id, do_update_settlements, do_update_table
from barker.views.voucher.update import check_permissions
from barker.views.voucher.void import do_void_settlements


@view_config(
    request_method="POST",
    route_name="v1_vouchers_id",
    renderer="json",
    request_param="split-bill",
    permission="Split Bill",
    trans=True,
)
def split_voucher(request):
    json = request.json_body
    now = datetime.now()
    id_ = uuid.UUID(request.matchdict["id"])
    update_table = request.GET["u"] == "true"
    item = request.dbsession.query(Voucher).filter(Voucher.id == id_).first()
    item.void = True
    item.void_reason = "Bill Split"
    do_void_settlements(item, request.dbsession)
    if update_table:
        request.dbsession.query(Overview).filter(
            Overview.voucher_id == item.id
        ).delete()

    inventories = [uuid.UUID(i) for i in json["inventories"]]

    one_inventories = [i for k in item.kots for i in k.inventories if i.id in inventories]
    two_inventories = [i for k in item.kots for i in k.inventories if i.id not in inventories]

    save(
        one_inventories,
        now,
        item.voucher_type,
        0,
        uuid.UUID(json["table"]["id"]),
        item.customer_id,
        update_table,
        uuid.UUID(request.authenticated_userid),
        request.effective_principals,
        request.dbsession,
    )
    save(
        two_inventories,
        now,
        item.voucher_type,
        item.pax,
        item.food_table_id,
        item.customer_id,
        update_table,
        uuid.UUID(request.authenticated_userid),
        request.effective_principals,
        request.dbsession,
    )

    transaction.commit()
    return True


def save(
    inventories,
    now,
    voucher_type,
    pax,
    table_id,
    customer_id,
    update_table,
    user_id,
    permissions,
    dbsession,
):
    product_quantities = {}
    skip_products = set()
    for i in inventories:
        if i.product_id in product_quantities:
            product_quantities[i.product_id] += i.quantity
        else:
            product_quantities[i.product_id] = i.quantity
    for product, quantity in product_quantities.items():
        if quantity < 0:
            raise ValidationError("Quantity is negative")
        elif quantity == 0:
            skip_products.add(product)
    check_permissions(None, voucher_type, permissions)
    bill_id = get_bill_id(voucher_type, dbsession)
    kot_id = dbsession.query(func.coalesce(func.max(Voucher.kot_id), 0) + 1).scalar()

    item = Voucher(
        now, pax, bill_id, kot_id, table_id, customer_id, voucher_type, user_id
    )
    dbsession.add(item)
    code = dbsession.query(func.coalesce(func.max(Kot.code), 0) + 1).scalar()
    kot = Kot(item.id, code, item.food_table_id, item.date, item.user_id)
    item.kots.append(kot)
    dbsession.add(kot)
    for index, old_inventory in enumerate(
        [i for i in inventories if i.product_id not in skip_products]
    ):
        inv = Inventory(
            kot.id,
            old_inventory.product_id,
            old_inventory.quantity,
            old_inventory.price,
            old_inventory.discount,
            old_inventory.is_happy_hour,
            old_inventory.tax_id,
            old_inventory.tax_rate,
            index,
        )
        kot.inventories.append(inv)
        dbsession.add(inv)
        for m in old_inventory.modifiers:
            mod = InventoryModifier(None, m.modifier_id, m.price)
            inv.modifiers.append(mod)
            dbsession.add(mod)
    do_update_settlements(item, dbsession)

    if len(kot.inventories) == 0:
        raise ValidationError("Please add some products!")

    if update_table:
        do_update_table(item, None, dbsession)
