import datetime
import uuid
from decimal import Decimal

import transaction
from pyramid.view import view_config
from sqlalchemy import func

from barker.models import (
    Voucher,
    Kot,
    Inventory,
    InventoryModifier,
    VoucherType,
    Product,
)
from barker.views.voucher import (
    get_tax,
    do_update_settlements,
    get_bill_id,
    do_update_table,
    check_permissions,
    get_guest_book,
)


@view_config(
    request_method="PUT", route_name="v1_vouchers_id", renderer="json", trans=True
)
def update(request):
    json = request.json_body
    now = datetime.datetime.now()
    update_table = request.GET["u"] == "true"
    voucher_type = VoucherType[request.GET["p"]]
    guest_book = get_guest_book(request.GET.get("g", None), request.dbsession)
    item = (
        request.dbsession.query(Voucher)
        .filter(Voucher.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )

    check_permissions(item, voucher_type, request.effective_principals)

    if guest_book is not None:
        item.pax = guest_book.pax
    item.food_table_id = json["table"]["id"]
    if "customer" in json and "id" in json["customer"]:
        item.customer_id = json["customer"]["id"]
    if item.voucher_type == VoucherType.KOT and voucher_type != VoucherType.KOT:
        item.date = now
        item.bill_id = get_bill_id(voucher_type, request.dbsession)
    item.voucher_type = voucher_type
    item.user_id = uuid.UUID(request.authenticated_userid)
    item.last_edit_date = now
    for k in item.kots:
        for i in k.inventories:
            i.tax_rate = get_tax(i.tax_rate, voucher_type)
            i.discount = next(
                round(Decimal(inv["discount"]), 5)
                for ko in json["kots"]
                for inv in ko["inventories"]
                if uuid.UUID(inv["id"]) == i.id
            )
    for k in (k for k in json["kots"] if "id" not in k and len(k["inventories"]) > 0):
        code = request.dbsession.query(func.coalesce(func.max(Kot.code), 0) + 1).scalar()
        kot = Kot(item.id, code, item.food_table_id, now, item.user_id)
        item.kots.append(kot)
        request.dbsession.add(kot)
        for index, i in enumerate(k["inventories"]):
            product = (
                request.dbsession.query(Product)
                .filter(Product.id == uuid.UUID(i["product"]["id"]))
                .first()
            )
            tax_rate = get_tax(product.sale_category.tax.rate, voucher_type)
            inv = Inventory(
                kot.id,
                product.id,
                round(Decimal(i["quantity"]), 2),
                product.price,
                round(Decimal(i["discount"]), 5),
                i["isHappyHour"],
                product.sale_category.tax_id,
                tax_rate,
                index,
            )
            kot.inventories.append(inv)
            request.dbsession.add(inv)
            for m in i["modifiers"]:
                mod = InventoryModifier(None, uuid.UUID(m["id"]), 0)
                inv.modifiers.append(mod)
                request.dbsession.add(mod)
    do_update_settlements(item, request.dbsession)
    if update_table:
        do_update_table(item, guest_book, request.dbsession)
    transaction.commit()
    return True
