import uuid

import transaction
from pyramid.view import view_config

from barker.models import Voucher, SettleOption, Settlement, Overview, VoucherType


@view_config(
    request_method="POST",
    route_name="v1_vouchers_id",
    renderer="json",
    request_param="void-bill",
    permission="Void Bill",
    trans=True,
)
def void_voucher(request):
    id_ = uuid.UUID(request.matchdict["id"])
    reason = request.json_body["reason"]
    update_table = request.GET["u"] == "true"

    item = request.dbsession.query(Voucher).filter(Voucher.id == id_).first()

    item.void = True
    item.void_reason = reason
    item.voucher_type = VoucherType.VOID

    do_void_settlements(item, request.dbsession)

    if update_table:
        request.dbsession.query(Overview).filter(
            Overview.voucher_id == item.id
        ).delete()
    transaction.commit()
    return True


def do_void_settlements(item, dbsession):
    settlements = []
    total_amount = item.amount
    settlements.append({"id": SettleOption.AMOUNT(), "amount": -total_amount})
    round_off = round(total_amount) - total_amount
    if round_off != 0:
        settlements.append({"id": SettleOption.ROUND_OFF(), "amount": -round_off})
    settlements.append({"id": SettleOption.VOID(), "amount": round(total_amount)})

    for i in settlements:
        amount = i["amount"]
        settlement_type_id = i["id"]
        old = [s for s in item.settlements if s.settled == settlement_type_id]
        if len(old) == 1:
            old[0].amount = amount
        else:
            s = Settlement(item.id, settlement_type_id, amount)
            item.settlements.append(s)
            dbsession.add(s)

    for i in (i for i in item.settlements if i.settled not in [x["id"] for x in settlements]):
        item.settlements.remove(i)
        dbsession.delete(i)
