import { Injectable } from '@angular/core';
import { ActivatedRouteSnapshot, CanActivate, Router, RouterStateSnapshot } from '@angular/router';
import { Observable } from 'rxjs/internal/Observable';
import { map } from 'rxjs/operators';
import { AuthService } from './auth.service';
import { ToasterService } from '../core/toaster.service';
import { User } from '../core/user';

@Injectable({
  providedIn: 'root'
})
export class AuthGuard implements CanActivate {

  constructor(private auth: AuthService, private router: Router, private toaster: ToasterService) {
  }

  checkUser(permission: string, user: User, state: RouterStateSnapshot): boolean {
    if (!user.isAuthenticated) {
      this.router.navigate(['login'], {queryParams: {returnUrl: state.url}});
      this.toaster.show('Danger', 'User is not authenticated');
      return false;
    }

    const hasPermission = this.auth.hasPermission(permission);
    if (!hasPermission) {
      this.toaster.show('Danger', 'You do not have the permission to access this area.');
    }
    return hasPermission;
  }

  canActivate(next: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | boolean {
    return this.auth.userObservable
      .pipe(
        map<User, boolean>((value: User) => this.checkUser(
          next.data['permission'], value, state
        ))
      );
  }
}
