import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Observable } from 'rxjs/internal/Observable';
import { catchError, map, tap } from 'rxjs/operators';
import { ErrorLoggerService } from '../core/error-logger.service';
import { User } from '../core/user';
import { merge, Subject } from 'rxjs';


const httpOptions = {
  headers: new HttpHeaders({'Content-Type': 'application/json'})
};

const loginUrl = '/v1/login';
const logoutUrl = '/logout';
const checkUrl = '/v1/auth';

@Injectable({
  providedIn: 'root'
})
export class AuthService {
  private readonly userSubject: Subject<User>;
  private user: User;
  public userObservable: Observable<User>;
  constructor(private http: HttpClient, private log: ErrorLoggerService) {
    this.userSubject = new Subject<User>();
    this.userObservable = merge(this.checkObserver(), this.userSubject).pipe(
      tap(x => this.user = x)
    );
  }

  checkObserver(): Observable<User> {
    return <Observable<User>>this.http.get<User>(checkUrl, httpOptions);
  }

  login(name: string, password: string, otp?: string, clientName?: string): Observable<any> {
    const data = {name: name, password: password};
    if (otp) {
      data['otp'] = otp;
    }

    if (clientName) {
      data['clientName'] = clientName;
    }

    return this.http.post(loginUrl, data, httpOptions)
      .pipe(
        tap((user: User) => this.userSubject.next(user)),
        catchError(this.log.handleError('AuthService', 'login'))
      );
  }

  logout(): Observable<boolean> {
    return <Observable<boolean>>this.http.post<User>(logoutUrl, {}, httpOptions)
      .pipe(
        tap((user: User) => this.userSubject.next(user)),
        map(() => true)
      );
  }

  hasPermission(permission: string): boolean {
    return this.user !== undefined && this.user.isAuthenticated && this.user.perms.indexOf(permission) !== -1;
  }
}
