import {Injectable} from '@angular/core';
import {HttpClient, HttpHeaders, HttpParams} from '@angular/common/http';
import {ErrorLoggerService} from '../core/error-logger.service';
import {catchError} from 'rxjs/operators';
import {Observable} from 'rxjs/internal/Observable';
import {Device} from '../core/device';

const httpOptions = {
  headers: new HttpHeaders({'Content-Type': 'application/json'})
};
const url = '/v1/devices';
const serviceName = 'DeviceService';

@Injectable({
  providedIn: 'root'
})
export class DeviceService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {
  }

  get(id: string): Observable<Device> {
    const getUrl: string = (id === null) ? `${url}/new` : `${url}/${id}`;
    return <Observable<Device>>this.http.get<Device>(getUrl)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`))
      );
  }

  list(): Observable<Device[]> {
    const options = {params: new HttpParams().set('l', '')};
    return <Observable<Device[]>>this.http.get<Device[]>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'list'))
      );
  }

  save(device: Device): Observable<Device> {
    return <Observable<Device>>this.http.post<Device>(`${url}/new`, device, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'save'))
      );
  }

  update(device: Device): Observable<Device> {
    return <Observable<Device>>this.http.put<Device>(`${url}/${device.id}`, device, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'update'))
      );
  }

  saveOrUpdate(device: Device): Observable<Device> {
    if (!device.id) {
      return this.save(device);
    } else {
      return this.update(device);
    }
  }

  delete(id: string): Observable<Device> {
    return <Observable<Device>>this.http.delete<Device>(`${url}/${id}`, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'delete'))
      );
  }
}
