import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { MatDialog } from '@angular/material';

import { MenuCategoryService } from '../menu-category.service';
import { MenuCategory } from '../../core/menu-category';
import { ToasterService } from '../../core/toaster.service';
import { ConfirmDialogComponent } from '../../shared/confirm-dialog/confirm-dialog.component';

@Component({
  selector: 'app-menu-category-detail',
  templateUrl: './menu-category-detail.component.html',
  styleUrls: ['./menu-category-detail.component.css']
})
export class MenuCategoryDetailComponent implements OnInit, AfterViewInit {
  @ViewChild('nameElement', { static: true }) nameElement: ElementRef;
  form: FormGroup;
  item: MenuCategory;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private fb: FormBuilder,
    private toaster: ToasterService,
    private ser: MenuCategoryService
  ) {
    this.createForm();
  }

  createForm() {
    this.form = this.fb.group({
      name: '',
      discountLimit: '',
      isActive: ''
    });
  }

  ngOnInit() {
    this.route.data
      .subscribe((data: { item: MenuCategory }) => {
        this.showItem(data.item);
      });
  }

  showItem(item: MenuCategory) {
    this.item = item;
    this.form.setValue({
      name: this.item.name,
      discountLimit: this.item.discountLimit,
      isActive: this.item.isActive
    });
  }

  ngAfterViewInit() {
    setTimeout(() => {
      this.nameElement.nativeElement.focus();
    }, 0);
  }

  save() {
    this.ser.saveOrUpdate(this.getItem())
      .subscribe(
        (result) => {
          this.toaster.show('Success', '');
          this.router.navigateByUrl('/menu-categories');
        },
        (error) => {
          this.toaster.show('Danger', error.error);
        }
      );
  }

  delete() {
    this.ser.delete(this.item.id)
      .subscribe(
        (result) => {
          this.toaster.show('Success', '');
          this.router.navigateByUrl('/menu-categories');
        },
        (error) => {
          this.toaster.show('Danger', error.error);
        }
      );
  }

  confirmDelete(): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: {title: 'Delete Menu Category?', content: 'Are you sure? This cannot be undone.'}
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete();
      }
    });
  }

  getItem(): MenuCategory {
    const formModel = this.form.value;
    this.item.name = formModel.name;
    this.item.discountLimit = +formModel.discountLimit;
    this.item.isActive = formModel.isActive;
    return this.item;
  }
}
