import { Component, OnInit, ViewChild } from '@angular/core';
import { MenuCategoryListDatasource } from './menu-category-list-datasource';
import { MenuCategory } from '../../core/menu-category';
import { ActivatedRoute, Router } from '@angular/router';
import { CdkDragDrop, moveItemInArray } from '@angular/cdk/drag-drop';
import { MatTable } from '@angular/material';
import { ToasterService } from '../../core/toaster.service';
import { MenuCategoryService } from '../menu-category.service';
import { BehaviorSubject } from 'rxjs';

@Component({
  selector: 'app-menu-category-list',
  templateUrl: './menu-category-list.component.html',
  styleUrls: ['./menu-category-list.component.css']
})
export class MenuCategoryListComponent implements OnInit {
  @ViewChild('table', { static: true }) table: MatTable<MenuCategory>;
  dataSource: MenuCategoryListDatasource;
  list: MenuCategory[];
  data: BehaviorSubject<MenuCategory[]>;
  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['name', 'discountLimit', 'isActive', 'isFixture'];

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private toaster: ToasterService,
    private ser: MenuCategoryService
  ) {
    this.data = new BehaviorSubject([]);
    this.data.subscribe((data: MenuCategory[]) => {
      this.list = data;
    });
  }

  ngOnInit() {
    this.route.data
      .subscribe((data: { list: MenuCategory[] }) => {
        data.list.forEach(x => x.discountLimit = x.discountLimit / 100);
        this.data.next(data.list);
      });
    this.dataSource = new MenuCategoryListDatasource(this.data);
  }

  updateSortOrder() {
    this.ser.updateSortOrder(this.list)
      .subscribe(
        (result) => {
          this.toaster.show('Success', '');
        },
        (error) => {
          this.toaster.show('Danger', error.error);
        }
      );
  }

  dropTable(event: CdkDragDrop<MenuCategory[]>) {
    const prevIndex = this.list.indexOf(event.item.data);
    moveItemInArray(this.list, prevIndex, event.currentIndex);
    this.data.next(this.list);
  }
}
