import {Injectable} from '@angular/core';
import {HttpClient, HttpHeaders, HttpParams} from '@angular/common/http';
import {ErrorLoggerService} from '../core/error-logger.service';
import {catchError} from 'rxjs/operators';
import {Observable} from 'rxjs/internal/Observable';
import {MenuCategory} from '../core/menu-category';

const httpOptions = {
  headers: new HttpHeaders({'Content-Type': 'application/json'})
};
const url = '/v1/menu-categories';
const serviceName = 'MenuCategoryService';

@Injectable({
  providedIn: 'root'
})
export class MenuCategoryService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {
  }

  get(id: string): Observable<MenuCategory> {
    const getUrl: string = (id === null) ? `${url}/new` : `${url}/${id}`;
    return <Observable<MenuCategory>>this.http.get<MenuCategory>(getUrl)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`))
      );
  }

  list(): Observable<MenuCategory[]> {
    const options = {params: new HttpParams().set('l', '')};
    return <Observable<MenuCategory[]>>this.http.get<MenuCategory[]>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'list'))
      );
  }

  save(menuCategory: MenuCategory): Observable<MenuCategory> {
    return <Observable<MenuCategory>>this.http.post<MenuCategory>(`${url}/new`, menuCategory, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'save'))
      );
  }

  update(menuCategory: MenuCategory): Observable<MenuCategory> {
    return <Observable<MenuCategory>>this.http.put<MenuCategory>(`${url}/${menuCategory.id}`, menuCategory, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'update'))
      );
  }

  updateSortOrder(list: MenuCategory[]): Observable<boolean> {
    return <Observable<boolean>>this.http.post<MenuCategory[]>(url, list, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'updateSortOrder'))
      );
  }

  saveOrUpdate(menuCategory: MenuCategory): Observable<MenuCategory> {
    if (!menuCategory.id) {
      return this.save(menuCategory);
    } else {
      return this.update(menuCategory);
    }
  }

  delete(id: string): Observable<MenuCategory> {
    return <Observable<MenuCategory>>this.http.delete<MenuCategory>(`${url}/${id}`, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'delete'))
      );
  }
}
