import {Injectable} from '@angular/core';
import {HttpClient, HttpHeaders, HttpParams} from '@angular/common/http';
import {ErrorLoggerService} from '../core/error-logger.service';
import {catchError} from 'rxjs/operators';
import {Observable} from 'rxjs/internal/Observable';
import {ModifierCategory} from '../core/modifier-category';

const httpOptions = {
  headers: new HttpHeaders({'Content-Type': 'application/json'})
};
const url = '/v1/modifier-categories';
const serviceName = 'ModifierCategoryService';

@Injectable({
  providedIn: 'root'
})
export class ModifierCategoryService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {
  }

  get(id: string): Observable<ModifierCategory> {
    const getUrl: string = (id === null) ? `${url}/new` : `${url}/${id}`;
    return <Observable<ModifierCategory>>this.http.get<ModifierCategory>(getUrl)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`))
      );
  }

  list(): Observable<ModifierCategory[]> {
    const options = {params: new HttpParams().set('l', '')};
    return <Observable<ModifierCategory[]>>this.http.get<ModifierCategory[]>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'list'))
      );
  }

  listIsActiveOfProduct(id: string): Observable<ModifierCategory[]> {
    const options = {params: new HttpParams().set('p', id).set('a', '')};
    return <Observable<ModifierCategory[]>>this.http.get<ModifierCategory[]>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'listIsActiveOfProduct'))
      );
  }

  save(modifierCategory: ModifierCategory): Observable<ModifierCategory> {
    return <Observable<ModifierCategory>>this.http.post<ModifierCategory>(`${url}/new`, modifierCategory, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'save'))
      );
  }

  update(modifierCategory: ModifierCategory): Observable<ModifierCategory> {
    return <Observable<ModifierCategory>>this.http.put<ModifierCategory>(`${url}/${modifierCategory.id}`, modifierCategory, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'update'))
      );
  }

  saveOrUpdate(modifierCategory: ModifierCategory): Observable<ModifierCategory> {
    if (!modifierCategory.id) {
      return this.save(modifierCategory);
    } else {
      return this.update(modifierCategory);
    }
  }

  delete(id: string): Observable<ModifierCategory> {
    return <Observable<ModifierCategory>>this.http.delete<ModifierCategory>(`${url}/${id}`, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'delete'))
      );
  }
}
