import { DataSource } from '@angular/cdk/collections';
import { map, tap } from 'rxjs/operators';
import { merge, Observable } from 'rxjs';
import { Modifier } from '../../core/modifier';


export class ModifierListDataSource extends DataSource<Modifier> {
  public data: Modifier[];
  public viewData: Modifier[];
  private filterValue: string;

  constructor(private readonly filter: Observable<string>, private readonly dataObs: Observable<Modifier[]>) {
    super();
    this.data = [];
    this.viewData = [];
    this.filter = filter.pipe(
      tap(x => this.filterValue = x)
    );
    this.dataObs = dataObs.pipe(
      tap(x => this.data = x)
    );
  }

  connect(): Observable<Modifier[]> {
    const dataMutations = [
      this.dataObs,
      this.filter
    ];

    return merge(...dataMutations).pipe(
      map((x: any) => {
        this.viewData = this.getFilteredData([...this.data]);
        return this.viewData;
      })
    );
  }

  disconnect() {
  }

  private getFilteredData(data: Modifier[]): Modifier[] {
    const filter = (this.filterValue === undefined) ? '' : this.filterValue;
      return data.filter(x => {
        return x.modifierCategory.id === filter || filter === '';
        }
      );
  }
}
