import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ModifierListDataSource } from './modifier-list-datasource';
import { MatTable } from '@angular/material';
import { Modifier } from '../../core/modifier';
import { BehaviorSubject } from 'rxjs';
import {ModifierCategory} from '../../core/modifier-category';

@Component({
  selector: 'app-modifier-list',
  templateUrl: './modifier-list.component.html',
  styleUrls: ['./modifier-list.component.css']
})
export class ModifierListComponent implements OnInit {
  @ViewChild('table', { static: true }) table: MatTable<Modifier>;
  dataSource: ModifierListDataSource;
  filter: BehaviorSubject<string>;
  form: FormGroup;
  list: Modifier[];
  data: BehaviorSubject<Modifier[]>;
  modifierCategories: ModifierCategory[];
  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns: string[] = ['name', 'showInBill', 'price', 'isActive', 'modifierCategory'];

  constructor(
    private route: ActivatedRoute,
    private fb: FormBuilder,
    private router: Router
  ) {
    this.form = this.fb.group({
      modifierCategory: ''
    });
    this.filter = new BehaviorSubject('');
    this.data = new BehaviorSubject([]);
    this.data.subscribe((data: Modifier[]) => {
      this.list = data;
    });
  }

  filterOn(val: any) {
    this.filter.next(val.value);
  }

  ngOnInit() {
    this.route.data
      .subscribe((data: { list: Modifier[], modifierCategories: ModifierCategory[] }) => {
        this.data.next(data.list);
        this.modifierCategories = data.modifierCategories;
      });
    this.dataSource = new ModifierListDataSource(this.filter, this.data);
  }
}
