import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { RouterModule, Routes } from '@angular/router';

import { ModifierListResolver } from './modifier-list-resolver.service';
import { ModifierResolver } from './modifier-resolver.service';
import { ModifierDetailComponent } from './modifier-detail/modifier-detail.component';
import { ModifierListComponent } from './modifier-list/modifier-list.component';

import { AuthGuard } from '../auth/auth-guard.service';
import { ModifierCategoryListResolver } from '../modifier-categories/modifier-category-list-resolver.service';

const modifierRoutes: Routes = [
  {
    path: '',
    component: ModifierListComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Modifiers'
    },
    resolve: {
      list: ModifierListResolver,
      modifierCategories: ModifierCategoryListResolver
    }
  },
  {
    path: 'new',
    component: ModifierDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Modifiers'
    },
    resolve: {
      item: ModifierResolver,
      modifierCategories: ModifierCategoryListResolver
    }
  },
  {
    path: ':id',
    component: ModifierDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Modifiers'
    },
    resolve: {
      item: ModifierResolver,
      modifierCategories: ModifierCategoryListResolver
    }
  }
];

@NgModule({
  imports: [
    CommonModule,
    RouterModule.forChild(modifierRoutes)

  ],
  exports: [
    RouterModule
  ],
  providers: [
    ModifierListResolver,
    ModifierResolver
  ]
})
export class ModifiersRoutingModule {
}
