import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { MatDialog } from '@angular/material';

import { SaleCategoryService } from '../sale-category.service';
import { SaleCategory } from '../../core/sale-category';
import { ToasterService } from '../../core/toaster.service';
import { ConfirmDialogComponent } from '../../shared/confirm-dialog/confirm-dialog.component';
import { Tax } from '../../core/tax';

@Component({
  selector: 'app-sale-category-detail',
  templateUrl: './sale-category-detail.component.html',
  styleUrls: ['./sale-category-detail.component.css']
})
export class SaleCategoryDetailComponent implements OnInit, AfterViewInit {
  @ViewChild('nameElement', { static: true }) nameElement: ElementRef;
  form: FormGroup;
  taxes: Tax[];
  item: SaleCategory;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private fb: FormBuilder,
    private toaster: ToasterService,
    private ser: SaleCategoryService
  ) {
    this.createForm();
  }

  createForm() {
    this.form = this.fb.group({
      name: '',
      tax: ''
    });
  }

  ngOnInit() {
    this.route.data
      .subscribe((data: { item: SaleCategory, taxes: Tax[] }) => {
        this.showItem(data.item);
        this.taxes = data.taxes;
      });
  }

  showItem(item: SaleCategory) {
    this.item = item;
    this.form.setValue({
      name: this.item.name,
      tax: this.item.tax.id ? this.item.tax.id : ''
    });
  }

  ngAfterViewInit() {
    setTimeout(() => {
      this.nameElement.nativeElement.focus();
    }, 0);
  }

  save() {
    this.ser.saveOrUpdate(this.getItem())
      .subscribe(
        (result) => {
          this.toaster.show('Success', '');
          this.router.navigateByUrl('/sale-categories');
        },
        (error) => {
          this.toaster.show('Danger', error.error);
        }
      );
  }

  delete() {
    this.ser.delete(this.item.id)
      .subscribe(
        (result) => {
          this.toaster.show('Success', '');
          this.router.navigateByUrl('/sale-categories');
        },
        (error) => {
          this.toaster.show('Danger', error.error);
        }
      );
  }

  confirmDelete(): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: {title: 'Delete Sale Category?', content: 'Are you sure? This cannot be undone.'}
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete();
      }
    });
  }

  getItem(): SaleCategory {
    const formModel = this.form.value;
    this.item.name = formModel.name;
    this.item.tax.id = formModel.tax;
    return this.item;
  }
}
