import {Injectable} from '@angular/core';
import {HttpClient, HttpHeaders, HttpParams} from '@angular/common/http';
import {ErrorLoggerService} from '../core/error-logger.service';
import {catchError} from 'rxjs/operators';
import {Observable} from 'rxjs/internal/Observable';
import {User} from '../core/user';

const httpOptions = {
  headers: new HttpHeaders({'Content-Type': 'application/json'})
};
const url = '/v1/users';
const serviceName = 'UserService';

@Injectable({
  providedIn: 'root'
})
export class UserService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {
  }

  get(id: string): Observable<User> {
    const getUrl: string = (id === null) ? `${url}/new` : `${url}/${id}`;
    return <Observable<User>>this.http.get<User>(getUrl)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`))
      );
  }

  list(): Observable<User[]> {
    const options = {params: new HttpParams().set('l', '')};
    return <Observable<User[]>>this.http.get<User[]>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'list'))
      );
  }

  listOfNames(): Observable<string[]> {
    const options = {params: new HttpParams().set('n', '')};
    return <Observable<string[]>>this.http.get<string[]>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'list'))
      );
  }

  save(user: User): Observable<User> {
    return <Observable<User>>this.http.post<User>(`${url}/new`, user, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'save'))
      );
  }

  update(user: User): Observable<User> {
    return <Observable<User>>this.http.put<User>(`${url}/${user.id}`, user, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'update'))
      );
  }

  saveOrUpdate(user: User): Observable<User> {
    if (!user.id) {
      return this.save(user);
    } else {
      return this.update(user);
    }
  }

  delete(id: string): Observable<User> {
    return <Observable<User>>this.http.delete<User>(`${url}/${id}`, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'delete'))
      );
  }
}
